#ifndef __SAPI4_TTS_TOOL_H_
#define __SAPI4_TTS_TOOL_H_

/*
*****************************************************
  Name:
    tts_tool.h

  Description:
      Functions for using SAPI4 compatible Text to Speech Synthesizers

  License:
      Copyright (c) 2002, Martin Goebbel
      All rights reserved.
      Redistribution and use in source and binary forms, with or without
      modification, are permitted provided that the following conditions
      are met:
        Redistributions of source code must retain the above copyright
        notice, this list of conditions and the following disclaimer.

        The names of its contributors may not be used to endorse or
        promote products derived from this software without specific
        prior written permission.

      THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
      IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
      TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
      PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE CONTRIBUTORS
      BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
      CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
      SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
      INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
      CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
      ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
      THE POSSIBILITY OF SUCH DAMAGE.

  History:
    Date        By       Comment
    23.11.2001  martin   Initial Version

*****************************************************
*/

/*
*****************************************************
SAPI4 uses COM in STA, so don't forget:
 - Call CoInitialize and CoUninitialize
 - The functions should only be called from the same Thread that has created the Objects
 - The Thread which has created the Objects needs a Message Loop

Other Caveats:
 - All output will be send to the default Audio Device ( WAVE_MAPPER )
 - The Callback Function ( "void (* TTSCALLBACK)(DWORD dwEvent)", see below ) is called from
   the COM Objects internal Thread.
 - Link with ole32.lib and uuid.lib
*****************************************************
*/

#include <windows.h>
#include <tchar.h>

#ifdef __cplusplus
extern "C" {
#endif

typedef struct _TtsModeInfoW
{
  GUID    gEngineID;
  WCHAR   szMfgName[262];
  WCHAR   szProductName[262];
  GUID    gModeID;
  WCHAR   szModeName[262];
  LANGID  LanguageID;
  WCHAR   szDialect[64];
  WCHAR   szSpeaker[262];
  WCHAR   szStyle[262];
  WORD    wGender;
  WORD    wAge;
  DWORD   dwFeatures;
  DWORD   dwInterfaces;
  DWORD   dwEngineFeatures;
} TTSMODEINFOW, *PTTSMODEINFOW;

#define TTS_EVENT_TEXTSTART 1
#define TTS_EVENT_TEXTDONE  2

typedef void *TTSENUM;
typedef void *TTS;
typedef void (* TTSCALLBACK)(DWORD dwEvent);

/*
  Enumeration of available Text to Speech Synthesizers

  Sample:

  TTSENUM en;
  TTSMODEINFOW info;

  if ( (en = tts_EnumCreate()) != NULL )
  {
    if ( tts_EnumReset(en) )
    {
      while ( tts_EnumNext(&info) )
      {
        ... Do something with info, e.g put it in a Listbox to make it selectable for an User
      }
    }

    tts_EndumDelete(en);
  }

*/
TTSENUM tts_EnumCreate ( void );
void    tts_EnumDelete ( TTSENUM ttsEnum );
BOOL    tts_EnumReset  ( TTSENUM ttsEnum );
BOOL    tts_EnumNext   ( TTSENUM ttsEnum, PTTSMODEINFOW pTtsModeInfo );


/*
  Text to Speech Functions

  Sample:

  TTS tts;

  ... Initialization
  if ( (tts = tts_Create(pInfo, NULL)) != NULL )
  {
    tts_TextOut(tts, _T("Initialization Successful"));
  }

  ... Shutdown
  tts_Delete(tts);

*/
TTS     tts_Create     ( PTTSMODEINFOW pTtsModeInfo, TTSCALLBACK lpfnCallback );
void    tts_Delete     ( TTS Tts );
BOOL    tts_Stop       ( TTS Tts );
BOOL    tts_Pause      ( TTS Tts );
BOOL    tts_Resume     ( TTS Tts );
BOOL    tts_TextOut    ( TTS Tts, LPCTSTR lpszText );

/*
  Read and Write Synthesizer Attributs
*/
BOOL    tts_SetVolume  ( TTS Tts, UINT nVolPercent );
BOOL    tts_GetVolume  ( TTS Tts, UINT *pVolPercent );
BOOL    tts_SetSpeed   ( TTS Tts, UINT nSpeedPercent );
BOOL    tts_GetSpeed   ( TTS Tts, UINT *pSpeedPercent );
BOOL    tts_SetPitch   ( TTS Tts, UINT nPitchPercent );
BOOL    tts_GetPitch   ( TTS Tts, UINT *pPitchPercent );

#ifdef __cplusplus
}
#endif

#endif
